<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco;

use Exception;
use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Geral\Entities\Banco;
use Geral\Entities\Empresa;
use Geral\Entities\Municipio;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Tipobaixapag;
use Fiscal\Entities\Notafiscal;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Financeiro\Util\Util;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;

/**
 * Class GeraCnabSantanderService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco
 */
class GeraCnabSantanderService extends GeraCnabService{

    const NUM_LOTE_HEADER_ARQUIVO = '0000';
    const ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    const ID_TP_ARQUIVO_HEADER_LOTE = '1';
    const ID_TP_ARQUIVO_SEGMENTO = '3';
    const ID_TP_ARQUIVO_TRAILER_LOTE = '5';
    const TIPO_INS_JUR_CPF = 1;
    const TIPO_INS_JUR_CNPJ = 2;
    const NOME_BANCO = 'Banco Santander';
    const CDBANCO = '033';
    const REMESSA = 1;
    const RETORNO = 2;
    const VERSAO_LAYOUT = '060';
    const TIPO_OPERACAO_CREDITO = 'C';
    const TIPO_SERVICO = '20';
    const VERSAO_LOTE = '031';
    const VERSAO_LOTE_TITULO_BANC = '030';
    const VERSAO_LOTE_TITULO_TRIBUTO = '010';
    const SEGMENTO_A = 'A';
    const EXCLUSAO = 'E';
    const INCLUSAO = 'I';
    const BLOQUEIO = 'B';
    const MOEDA = 'BRL';
    const SEGMETO_B = 'B';
    const LOTE_SERVICO_TRAILER_ARQUIVO = '9999';
    const ID_TP_ARQUIVO_TRAILER = '9';
    const SEGMENTO_J = 'J';
    const SEGMENTO_J52 = '52';
    const SEGMENTO_O = 'O';
    const SEGMETO_I = 'I';

    private $num_header_lote = 0;
    private $seqRegLote = 0;
    private $qtdRegiLote = 0;  //Qtde de registro do lote
    private $qtdRegiArq = 0; //Quantidade de registro do arquivo
    private $qtLote = 0; //Quantidade de Lotes no arquivo
    private $vrPagtLote = 0.00; // Somatório dos valores de inclusao do lote;
    private $formapgto;

    /**
     * @param Cnabpag $remessa
     * @throws Exception
     */
    public function __construct(Cnabpag $remessa){

        try {
            $this->nrorg = $remessa->getNrorg();

            $this->remessa = $remessa;

            // Carrega os atributos Filial e Empresa
            $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
            $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

            // Busca dados referente a filial se foi informada
            if ($remessa->getCdfilial()) {
                if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->filial->getNrinsjurfili();
                $this->nomeempresa = $this->filial->getNmfilial();
            } // Se nao busca dados da Empresa
            else {
                if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->empresa->getNrinsjurempr();
                $this->nomeempresa = $this->empresa->getNmempresa();
            }
            /** @var contcorr Contcorr */
            $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Bradesco
     * @throws \Exception
     * @internal param $remessa
     * @return mixed
     */
    public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null){
        try{

            $file = fopen($caminhoArquivo.'.txt', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);

            // Agrupar boletos por tipo de baixa e por banco favorecido
            $boletosAgrup = self::agrupaBoletos($boletos, $file);

            //Percorre as formas de pagamento
            foreach($boletosAgrup as $segmento => $boletosFormaPgo){
                //Gera Segmentos AB
                if($segmento == 'SEGMENTOAB'){

                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLote($key);
                        fprintf($file, "%s",$headerlote);

                        foreach($boletosBco as $boleto){
                            if($this->remessa->getIdprenegociado() == 'S'){
                                $segmentoI = self::geraDetalheSegmentoI($boleto);
                                fprintf($file, "%s",$segmentoI);
                            }
                            $segmentoA = self::geraDetalheSegmentoA($boleto, $boleto->getCdbancbolp());
                            $segmentoB = self::geraDetalheSegmentoB($boleto);
                            fprintf($file, "%s",$segmentoA);
                            fprintf($file, "%s",$segmentoB);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                // Fim Segmento AB

                //Gera Segmentos J
                if($segmento == 'SEGMENTOJ'){
                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLoteBoletoBanc($key);
                        fprintf($file, "%s",$headerlote);
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                            /** @var $boleto Boletpagar */
                            $segmentoJ = self::geraSegmentoJ($boleto);
                            fprintf($file, "%s",$segmentoJ);
                            /** @var $boleto Boletpagar */
                            $segmentoJ = self::geraSegmentoJ52($boleto);
                            fprintf($file, "%s",$segmentoJ);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                } // Fim Segmento J

                //Gera Segmentos O
                if($segmento == 'SEGMENTOO'){
                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLoteTributos($key);
                        fprintf($file, "%s",$headerlote);
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                            /** @var $boleto Boletpagar */
                            $segmentoO = self::geraSegmentoO($boleto);
                            fprintf($file, "%s",$segmentoO);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                } // Fim Segmento O
            }
            $trailerArquivo = self::geraTrailerArquivo();
            fprintf($file, "%s",$trailerArquivo);
            fclose($file);
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }

    /** Retorna array de Boletos Agrupados por Forma de Pagamento e por código do banco favorecido*/
    private function agrupaBoletos($boletos){
        $boletosAgroup = array();

        $segmentosAB = array(Tipobaixapag::TIPOBAIXA_CNAB,
            Tipobaixapag::TIPOBAIXA_CNAB_DOC,
            Tipobaixapag::TIPOBAIXA_TED_CIP,
            Tipobaixapag::TIPO_BAIXA_TED_STR,
        );

        $segmentoJ = array(Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

        $segmentoO = array(Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS, Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL, Tipobaixapag::TIPO_BAIXA_FGTS);

        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){

            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag(
                $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']
            );

            /** @var Tipobaixapag $tipobaixapag */
            $tipobaixapag = Tipobaixapag::findOneByNrorgAndCdtpbaixapag($boleto->getNrorg(), $boleto->getCdtpbaixapag());

            if($tipobaixapag)
                $idbaixautilp = $tipobaixapag->getIdbaixautilp();
            else $idbaixautilp = null;

            if(in_array($idbaixautilp,$segmentosAB)){

                if(self::getBanco($boleto) == GeraCnabService::CODIGO_COMPENSACAO_SANTANDER)
                    $boletosAgroup['SEGMENTOAB'] [GeraCnabService::CODIGO_COMPENSACAO_SANTANDER] [] = $boleto;
                else
                    $boletosAgroup['SEGMENTOAB'] ['OUTROS_BANCOS'] [] = $boleto;

            }else{
                if(in_array($idbaixautilp,$segmentoJ)){
                    $cdbanco = self::getBanco($boleto,Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

                    if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_SANTANDER)){
                        $boletosAgroup['SEGMENTOJ'] [GeraCnabService::CODIGO_COMPENSACAO_SANTANDER] [] = $boleto;
                    }
                    else{
                        $boletosAgroup['SEGMENTOJ'] ['OUTROS_BANCOS'] [] = $boleto;
                    }

                }else{
                    if(in_array($idbaixautilp,$segmentoO)){
                        $cdbanco = self::getBanco($boleto, $idbaixautilp);

                        if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_SANTANDER)){
                            $boletosAgroup['SEGMENTOO'] [GeraCnabService::CODIGO_COMPENSACAO_SANTANDER] [] = $boleto;
                        }
                        else{
                            $boletosAgroup['SEGMENTOO'] ['OUTROS_BANCOS'] [] = $boleto;
                        }

                    }else{
                        throw new \Exception ('Tipo de Baixa não reconhecido');
                    }
                }
            }
        }
        return $boletosAgroup;
    }


    // Registro Header de Arquivo
    /**
     * @throws Exception
     * @return string
     */
    public function geraHeaderArquivo(){

        try {

            $this->qtdRegiArq = 0;
            $this->qtdRegiArq++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= self::NUM_LOTE_HEADER_ARQUIVO;

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_ARQUIVO;

            //Filler - Posição: 9 a 17 - Picture: X(9)
            $linha.= self::preenche_brancos('',9);

            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            //Código do convênio no banco
            //Posição: 33 a 52
            //Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= self::preenche_brancos($identpag,20);
            }

            //Número da agência centralizadora - Posição: 53 a 57 - Picture: 9(05)
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            
            if($digitoAgencia){
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            }
                

            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da conta - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
            $linha.= ' ';

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Nome do Banco - Posição: 103 a 132 - Picture: X(30)
            $nmbanco = self::NOME_BANCO;
            $linha.= self::preenche_brancos($nmbanco,30);


            //Filler - Posição: 133 a 142 - Picture: X(10)
            $linha.= self::preenche_brancos('',10);

            //Código remessa/Retorno (1 = Remessa, 2 = Retorno) - Posição: 143 a 143 - Picture: 9(1)
            $linha.= self::REMESSA;

            //Data da geração do arquivo - Posição: 144 a 151 - Picture: 9(8)
            $linha.= self::getDataGeracao();

            //Hora de geração do arquvio - Posição: 152 a 157 - Picture 9(6)
            $linha.= self::getHorarioAtual();

            //Número de sequência do arquivo - Posição: 158 a 163 - Picture: 9(06)
            $linha.= substr($this->remessa->getNrseqremessapag(),-6);

            //Número da versão do layout do arquivo - Posição: 164 a 166 - Picture: 9(03)
            $linha.= self::VERSAO_LAYOUT;

            //Densidade de gravação do arquivo - Posição: 167 a 171 - Picture: 9(5)
            $linha.= str_repeat('0',5);

            //Campo para uso exclusivo do banco - Posição: 172 a 191 - Picture: X(20)
            //Campo para uso exclusivo da empresa - Posição: 172 a 191 - Picture: X(20)
            //Filler - Posição: 212 a 230 - Picture X(19)
            //Ocorrência - Posição 131 - 240 - Picture X(10)
            $linha.= str_repeat(' ',69);

            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdbanco
     * @throws \Exception
     * @internal param $formaLancamento
     * @return string
     */
    public function geraHeaderLote($cdbanco, $boletpagar = null){

        try{
            $this->vrPagtLote = 0;
            $this->qtdRegiLote = 0;
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
            $this->qtLote++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= str_pad(++$this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_LOTE;

            //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(01)
            $linha.= self::TIPO_OPERACAO_CREDITO;

            //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(02)
            $linha.= self::TIPO_SERVICO;

            //Forma de lançamento - Posição: 12 a 13 - Picture: 9(02)
            if($cdbanco == self::CODIGO_COMPENSACAO_SANTANDER)
                $this->formapgto = '01';
            else
                $this->formapgto = '03';
            $linha.= $this->formapgto;

            //Número da versão no layout do lote - Posição: 14 a 16 - Picture: 9(03)
            $linha.= self::VERSAO_LOTE;

            //Filler - Posição: 17 a 17 - Picture: X(01)
            $linha .= ' ';

            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            //Código do convênio no banco
            //Posição: 33 a 52
            //Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= self::preenche_brancos($identpag,20);
            }

            //Número da agência centralizadora - Posição: 53 a 57 - Picture: 9(05)
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            
            if($digitoAgencia){
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            }
            
            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da conta - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
            $linha.= ' ';

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Campo de mensagem genérica - Posição: 103 a 142 - Picture: X(40)
            //Filler - Posição: 143 a 230 - Picture: X(88)
            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',138);
            return $linha."\r\n";


        }catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * @param $cdbanco
     * @throws \Exception
     * @internal param $formaLancamento
     * @return string
     */
    public function geraHeaderLoteBoletoBanc($cdbanco){

        try{
            $this->vrPagtLote = 0;
            $this->qtdRegiLote = 0;
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
            $this->qtLote++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= str_pad(++$this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_LOTE;

            //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(01)
            $linha.= self::TIPO_OPERACAO_CREDITO;

            //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(02)
            $linha.= self::TIPO_SERVICO;

            //Forma de lançamento - Posição: 12 a 13 - Picture: 9(02)
            $this->formapgto = '';
            if($cdbanco == self::CODIGO_COMPENSACAO_SANTANDER)
                $this->formapgto = '30';
            else
                $this->formapgto = '31';

            $linha.= $this->formapgto;

            //Número da versão no layout do lote - Posição: 14 a 16 - Picture: 9(03)
            $linha.= self::VERSAO_LOTE_TITULO_BANC;

            //Filler - Posição: 17 a 17 - Picture: X(01)
            $linha .= ' ';

            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            //Código do convênio no banco
            //Posição: 33 a 52
            //Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= self::preenche_brancos($identpag,20);
            }

            //Número da agência centralizadora - Posição: 53 a 57 - Picture: 9(05)
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            
            if($digitoAgencia){
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            }
            //$linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);

            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da conta - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
            $linha.= ' ';

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Filler - Posição: 103 a 142 - Picture: X(40)
            $linha.= str_repeat(' ',40);

            //Endereço - Posição: 143 a 172 - Picture: X(30)
            $linha.= str_repeat(' ',30);

            //Nº Endereço - Posição: 173 a 177 - Picture: X(5)
            $linha.= str_repeat('0',5);

            //Complemento Endereço - Posição: 178 a 192 - Picture: X(15)
            $linha.= str_repeat(' ',15);

            //CIDADE - Posição: 193 a 212 - Picture: X(20)
            $linha.= str_repeat(' ',20);

            //CEP - Posição: 213 a 220 - Picture: X(8)
            $linha.= str_repeat('0',8);

            //UF - Posição: 221 a 222 - Picture: X(2)
            $linha.= str_repeat(' ',2);

            //FILLER - Posição: 223 a 230 - Picture: X(8)
            $linha.= str_repeat(' ',8);

            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha."\r\n";


        }catch(\Exception $e){
            throw $e;
        }
    }


    public function geraHeaderLoteTributos($cdbanco){

        try{
            $this->vrPagtLote = 0;
            $this->qtdRegiLote = 0;
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
            $this->qtLote++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= str_pad(++$this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_LOTE;

            //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(01)
            $linha.= self::TIPO_OPERACAO_CREDITO;

            //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(02)
            $linha.= self::TIPO_SERVICO;

            //Forma de lançamento - Posição: 12 a 13 - Picture: 9(02)
            $linha.= '11';

            //Número da versão no layout do lote - Posição: 14 a 16 - Picture: 9(03)
            $linha.= self::VERSAO_LOTE_TITULO_TRIBUTO;

            //Filler - Posição: 17 a 17 - Picture: X(01)
            $linha .= ' ';

            if ($this->remessa->getCdfilial()){
                if ($this->filial->getIdtpijurfili() == 'F'){
                    $linha .= '1';
                }else{
                    $linha .= '2';
                }
                $nrinsjurfili = trim($this->filial->getNrinsjurfili());
                if (strlen($nrinsjurfili) > 14){
                    $msg = "Código da inscrição da filial deve ter no máximo 14 posições.";
                }
                $linha .= $nrinsjurfili;
                $nmemprfili = str_pad($this->filial->getNmfilial(), 30);
            }else{
                if ($this->empresa->getIdtpijurempr() == 'F'){
                    $linha .= '1';
                }else{
                    $linha .= '2';
                }
                $nrinsjurempr = trim($this->empresa->getNrinsjurEmpr());
                if (strlen($nrinsjurempr) > 14){
                   $msg = "Código da inscrição da empresa deve ter no máximo 14 posições.";
               }
               $linha .= $this->empresa->getNrinsjurempr();
               $nmemprfili = str_pad($this->empresa->getNmempresa(), 30);
            }



            //Código do convênio no banco
            //Posição: 33 a 52
            //Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= self::preenche_brancos($identpag,20);
            }

            //Número da agência centralizadora - Posição: 53 a 57 - Picture: 9(05)
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            
            if($digitoAgencia){
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            }
            
            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da conta - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
            $linha.= ' ';

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Filler - Posição: 103 a 142 - Picture: X(40)
            $linha.= str_repeat(' ',40);

            //Endereço - Posição: 143 a 172 - Picture: X(30)
            $linha.= str_repeat(' ',30);

            //Nº Endereço - Posição: 173 a 177 - Picture: X(5)
            $linha.= str_repeat(' ',5);

            //Complemento Endereço - Posição: 178 a 192 - Picture: X(15)
            $linha.= str_repeat(' ',15);

            //CIDADE - Posição: 193 a 212 - Picture: X(20)
            $linha.= str_repeat(' ',20);

            //CEP - Posição: 213 a 220 - Picture: X(8)
            $linha.= str_repeat(' ',8);

            //UF - Posição: 221 a 222 - Picture: X(2)
            $linha.= str_repeat(' ',2);

            //FILLER - Posição: 223 a 230 - Picture: X(8)
            $linha.= str_repeat(' ',8);

            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha."\r\n";


        }catch(\Exception $e){
            throw $e;
        }
    }


    /**
     * retorna a data atual no formato ddmmYYYY
     * @return string
     */
    private function getDataGeracao(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('d').$dataatual->format('m').$dataatual->format('Y');
    }

    /**
     * @return string
     */
    private function getHorarioAtual(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('H').$dataatual->format('i').$dataatual->format('s');
    }


    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
     */
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }


    /**
     * @param \Fiscal\Entities\Boletpagar $boleto
     * @param $cdbanco
     * @throws Exception
     * @internal param $formapgto
     * @internal param array $boletos
     * @return string
     */
    public function geraDetalheSegmentoA(Boletpagar $boleto, $cdbanco){
        try {

            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMENTO_A;

            //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
            if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha .= '9';  // Exclusão
            }
            else{
                $linha.=  '0';  // Inclusão e Bloqueio
                //Somatório dos registros com tipo de movimento = 0 (inclusão)
                $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            }

            //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
            if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
                $linha .= '00';  //INCLUSAO
            }
            else{
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.=  '99';  // EXLUSAO
                }
                else{//BLoqueio
                    $linha.=  '09';
                }
            }
            //Busca tipo de baixa
            /** @var $tipobaixapag Tipobaixapag */
            $tipobaixapag = Tipobaixapag::findOneByCdtpbaixapagAndNrorg($boleto->getCdtpbaixapag(),
                $this->nrorg );

            //Código da câmara de compensação - Posição: 18 a 20 - Picture: 9(03)
            if($this->formapgto == '01' or $this->formapgto == '03'){
                if($cdbanco != self::CODIGO_COMPENSACAO_SANTANDER){

                    if(floatval($boleto->getVrfinalbpag()) > floatval($tipobaixapag->getVrmaxdoccnab())){
                        if($tipobaixapag->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO)
                            $linha.= '018'; //TED CIP
                        else
                            $linha.= '810'; //TED STR
                    }
                    else {
                        $linha.= '700'; //DOC
                    }
                }else{
                    $linha.= '000'; //CC
                }

                //Código do banco do favorecido - Posição: 21 a 23 - Picture: 9(03)
                $linha.= str_pad($cdbanco,3,'0',STR_PAD_LEFT);

                if($tipobaixapag->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG){
                    
                    //Número da agência mantenedora - Posição: 24 a 28 - Picture: 9(05)
                    $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
                    $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
                    
                    if($digitoAgencia){
                        $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
                    } else {
                        $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
                    }
                    
                    //Dídito verificador da agência - Posição: 29 a 29 - Picture: X(01)
                    $linha.= ' ';

                    //Número da conta - Posição: 30 a 41 - Picture: 9(12)
                    $linha.= self::preenche_brancos('',12);

                    //Dídito verificador da conta - Posição: 42 a 42 - Picture: X(01)
                    $linha.= '0';

                    //Dídito verificador da agência/conta - Posição: 43 a 43 - Picture: X(01)
                    $linha.= ' ';
                }
                else{
                    //Busca a máscara da CONTA e AGÊNCIA do fornecedor cadastrada na tela de Banco
                    /** @var $bancoFornecedor Banco */
                    $bancoFornecedor = self::getBancoForn($boleto->getCdbancbolp(), $boleto->getNrorg());

                    //{========= Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado com as máscaras,
                    // entrará nesse if =========}
                    if($bancoFornecedor and $bancoFornecedor->getCdpictagen()){
                        //{ se na máscara da agência NÃO tem '-' }
                        if(strpos($bancoFornecedor->getCdpictagen(),'-') === false){
                            //Agencia
                            $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,5)
                                ,5,'0',STR_PAD_LEFT);
                            //DV
                            $linha.= ' ';
                        }
                        else{
                            //{ Caso a máscara da agência TENHA '-' }
                            //Agencia
                            $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,
                                    strpos($bancoFornecedor->getCdpictagen(),'-'))
                                ,5,'0',STR_PAD_LEFT);

                            //Se foi cadastrada a agencia do favorecido como 9999 na tela de boleto e
                            //tempos depois foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                            //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema.
                            if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                    ,strpos($bancoFornecedor->getCdpictagen(),'-')))>0){

                                //DV
                                $pos = strpos($bancoFornecedor->getCdpictagen(),'-');
                                $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                    ,$pos,1);
                            }
                            else{
                                $linha.= ' ';
                            }
                        }
                    }
                    else{
                        //{======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema,
                        //NÃO parametrizado com as máscaras, entrará nesse else ====}
                        if(strpos($boleto->getCdagenbolp(),'-')=== false){
                            // { se o que foi digitado no campo de agencia do favorecido da tela de boleto NÃO tem '-' }
                            //Agencia
                            $linha.= str_pad(substr(trim($boleto->getCdagenbolp()),0,5),5,'0',STR_PAD_LEFT);

                            //DV
                            $linha.= ' ';
                        }
                        else{
                            //{ caso o que foi digitado no campo de agencia do favorecido da tela de boleto TENHA '-' }
                            //Agencia
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdagenbolp(),0,
                                strpos($boleto->getCdagenbolp(),'-')
                            )),5,'0',STR_PAD_LEFT);

                            //DV
                            $pos = strpos($boleto->getCdagenbolp(),'-') +1;
                            $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                ,$pos,1);
                        }
                    }

                    //Número da conta corrente favorecido - Posição: 030 a 041 - Picture: 9(11)
                    //MAIS
                    //Dídito verificador da conta - Posição: 42 a 42 - Picture: X(01)

                    //{========== Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado
                    //com as máscaras, entrará nesse if ===========}
                    if($bancoFornecedor and $bancoFornecedor->getCdpictcont()){
                        //{ se na máscara da conta corrente TEM '-' }
                        if(strpos($bancoFornecedor->getCdpictcont(),'-')){

                            //{ tamDifVer só pode assumir os valor 1 ou 2.
                            // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                            // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 }

                            $tamDigVer = strlen(trim(substr($bancoFornecedor->getCdpictcont(),
                                strpos($bancoFornecedor->getCdpictcont(),'-')+1,
                                2)));
                            if($tamDigVer>1){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        strpos($bancoFornecedor->getCdpictcont(),'-'))),
                                    12,'0',STR_PAD_LEFT);

                                //Se foi cadastrada a cc do favorecido como 9999 na tela de boleto e tempos depois
                                //foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                                //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema
                                if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                        ,strpos($bancoFornecedor->getCdpictcont(),'-')))>0){
                                    //DV
                                    $pos = strpos($bancoFornecedor->getCdpictcont(),'-')+1 ;
                                    $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                        ,$pos,1);
                                }
                                else{
                                    $linha.= ' ';
                                }

                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                    -1
                                )),12,'0',STR_PAD_LEFT);

                                //DV
                                $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp())),
                                    -1);

                            }

                        }
                        else{
                            //{ CASO a máscara da conta corrente NÃO tenha '-' }
                            if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_SANTANDER){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                                ),12,'0',STR_PAD_LEFT);

                                //DV
                                $linha.= substr($boleto->getCdccorbolp(),-1);

                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                                ),12,'0',STR_PAD_LEFT);

                                //DV
                                $linha.= ' ';
                            }
                        }

                    }
                    // {======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema, NÃO parametrizado com
                    //as máscaras, entrará nesse else ====}
                    else{
                        if(strpos($boleto->getCdccorbolp(),'-')){
                            //se o que foi digitado no campo de conta Corrente do favorecido da tela de boleto tem '-'
                            //CC

                            //{ tamDifVer só pode assumir os valor 1 ou 2.
                            // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                            // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 }

                            $tamDigVer = strlen(trim(substr($boleto->getCdccorbolp(),
                                strpos($boleto->getCdccorbolp(),'-')+1,
                                2)));

                            if($tamDigVer>1){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        strpos($boleto->getCdccorbolp(),'-'))
                                ),11,'0',STR_PAD_LEFT);
                                $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+1,1));
                                //DV
                                $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+2,1));
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        strpos($boleto->getCdccorbolp(),'-'))
                                ),12,'0',STR_PAD_LEFT);

                                //DV
                                $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+1));
                            }
                        }
                        //CASO o que foi digitado no campo de conta corrente do favorecido da tela de
                        //Boleto NÃO tenha '-'
                        else{
                            if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_SANTANDER){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                                ),12,'0',STR_PAD_LEFT);

                                //DV
                                $linha.= substr($boleto->getCdccorbolp(),-1);
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                                ),12,'0',STR_PAD_LEFT);

                                //DV
                                $linha.= ' ';
                            }
                        }
                    }
                    //Dídito verificador da agência/conta - Posição: 43 a 43 - Picture: X(01)
                    $linha.= ' ';
                }
                //Nome do favorecido - Posição: 44 a 73 - Picture: X(30)
                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

                /** @var $contaPagar Contapagar */
                //Número do documento atribuído pela empresa - Posição: 74 a 93 - Picture: X(20)
                $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                    $boleto->getCdfilial(),
                    $boleto->getNrorg());
                $nrnotafisc = $contaPagar->getLanctonf()->getNrnotafisc();
                if($nrnotafisc){
                    $linha.= str_pad($nrnotafisc,20,' ',STR_PAD_RIGHT);
                } else{
                    $linha.= self::preenche_brancos(' ',20);
                }

                //Data para lançamento - Posição: 94 a 101 - Picture: 9(08)
                $linha.= $boleto->getDtvenbolpag()->format('dmY');

                //Tipo da Moeda - Posição: 102 a 104 - Picture: X(03)
                $linha.= self::MOEDA;

                //Quantidade da moeda - Posição: 105 a 119 - Picture: 9(15)
                $linha.= str_pad('',15,'0', STR_PAD_RIGHT);

                //Valor do lancamento - Posição: 120 a 134 - Picture: 9(13)V9(2)
                $linha.= str_pad(str_replace(array('.',','),'',
                    number_format($boleto->getVrfinalbpag(),2)
                ),15,'0',STR_PAD_LEFT);
                //Número do documento atribuido pelo banco - Posição: 135 a 154 - Picture: X(20)
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.= str_pad(substr($boleto->getNrretcnabpag(),0,20),20,' ',STR_PAD_RIGHT);
                } else{
                    $linha.= self::preenche_brancos(' ',20);
                }

                //Data da efetivação do lancamento - Posição: 155 a 162 - Picture: 9(08)
                $linha.= str_pad('',8,'0');

                //Valor do lançamento - Posição: 163 a 177 - Picture: 9(15)
                $linha.= str_pad('',15,'0');

                //Outras Informações - Posição: 178 a 217 - Picture: X(40)
                $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),40,' ',STR_PAD_RIGHT);

                //Código da finalidade - Posição: 218 a 219 - Picture: 9(02)
                if($this->formapgto == '03')
                    $linha.= '03';
                else
                    $linha.= '99';

                //Filler - Posição: 220 a 229 - Picture: X(10)
                $linha.= self::preenche_brancos(' ',10);

                //Emissão de aviso - Posição: 230 a 230 - Picture: X(01)
                $linha.= '0';

                //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
                $linha.= self::preenche_brancos(' ',10);
            }
            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrlanbolpag
     * @param $cdfilibpag
     * @param $nrorg
     * @return Contapagar
     */
    private function getContaPagar($nrlanbolpag, $cdfilibpag, $nrorg){
        /** @var $contaPagar Contapagar */
        $contaPagar = Contapagar::findOneByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag, $cdfilibpag, $nrorg);
        return $contaPagar;
    }
    
    /**
     * * @param $nrlanctonf
     * @param $cdfilial
     * @param $nrorg
     * @return Notafiscal
     */
    private function getNotaFiscal($nrlanctonf, $cdfilial, $nrorg){
        /** @var $notafiscal Notafiscal */
        $notafiscal = Notafiscal::findOneByNrlanctonfAndCdfilialAndNrorg($nrlanctonf, $cdfilial, $nrorg);
        return $notafiscal;
    }

    /**
     * @param Boletpagar $boleto
     * @throws Exception
     * @return string
     * @internal param $cdbanco
     */
    public function geraDetalheSegmentoB(Boletpagar $boleto){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SANTANDER;

            //Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMETO_B;

            //Filler - Posição: 15 a 17 - Picture: 9(03)
            $linha.= str_repeat(' ',3);
            
            //Tipo de incrição do fornecedor - Posição: 18 a 18 - Picture: 9(01)
            //Código do CPF/CNPJ - Posição: 19 a 32 - Picture: 9(14)
            if($boleto->getNrinscnabbop()){
                if(strlen(Util::campoCnabValido($boleto->getNrinscnabbop())) == 11){
                    $linha.= self::TIPO_INS_JUR_CPF;
                }else{
                    $linha.= self::TIPO_INS_JUR_CNPJ;
                }
                $linha.= str_pad(substr($boleto->getNrinscnabbop(),0,14),14,'0', STR_PAD_LEFT);
            }else{
                if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
                    $linha.= self::TIPO_INS_JUR_CPF;
                }else{
                    $linha.= self::TIPO_INS_JUR_CNPJ;
                }
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),14,'0', STR_PAD_LEFT);
            }


            //Nome do logradouro do favorecido - Posição: 33 a 62 - Picture: X(30)
            $endforn = $boleto->getForneced()->getEndefornecobr();
            if(! $endforn)
                $endforn = $boleto->getForneced()->getEndeforneprinc();
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getDsendeforn()),0,30),30,' ',STR_PAD_RIGHT);

            //Número do imóvel do favorecido - Posição: 63 a 67 - Picture: 9(05)
            $linha.= str_repeat('0',5);

            //Complemento do imóvel - Posição: 68 a 82 - Picture: X(15)
            $linha.= str_repeat(' ',15);

            //Bairro do favorecido - Posição: 83 a 97 - Picture: 9(15)
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getNmbairforn()),0,15),15,' ',STR_PAD_RIGHT);

            //Município do favorecido - Posição: 98 a 117 - Picture: X(20)
            $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($this->empresa->getSgestado(), $this->empresa->getCdpais(), $this->empresa->getCdmunicipio());
            $linha.= str_pad($municipio->getNmmunicipio(), 20);

            //CEP - Posição: 118 a 122 - Picture: 9(05)
            if($endforn->getNrcepforn()){
                $linha.= str_pad(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),0,5),5,'0');
            }else{
                $linha.= str_repeat('0',5);
            }

            //Complemento do CEP - Posição: 123 a 125 - Picture: X(03)
            if(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3)){
                $linha.= substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3);

            }else{
                $linha.= str_repeat(' ',3);
            }

            //Estado - Posição: 126 a 127 - Picture: X(02)
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getSgestado()),0,2),2,' ',STR_PAD_RIGHT);

            //Data do vencimento (Nominal) - Posição: 128 a 135 - Picture: 9(08)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do documento (Nominal) - Posição: 136 a 150 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Valor do abatimento - Posição: 151 a 165 - Picture: 9(13)V9(2)
            $linha .= str_repeat('0', 15);

            //Valor do desconto - Posição: 166 a 180 - Picture: 9(13)V9(2)
            $linha .= str_repeat('0', 15);

            //Valor da mora - Posição: 181 a 195 - Picture: 9(13)V9(2)
            $linha .= str_repeat('0', 15);

            //Valor da multa - Posição: 196 a 210 - Picture: 9(13)V9(2)
            $linha .= str_repeat('0', 15);

            //Horário de envio do TED STR - Posição: 211 a 214 - Picture: X(04)
            $linha .= str_repeat('0', 4);

            //Filler - Posição: 215 a 240 - Picture: X(26)
            $linha .= str_repeat(' ', 26);

            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @internal param $cdbanco
     * @throws Exception
     * @return string
     * @internal param $caminho
     * @internal param $nrorg
     */
    public function geraTraillerLote($somavalores = null){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
            $linha = self::CODIGO_COMPENSACAO_SANTANDER;

            //Número de sequência do lote - Posição: 4 a 7 - Picture: 9(4)
            $linha .= str_pad($this->num_header_lote, 4, '0', STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha .= self::ID_TP_ARQUIVO_TRAILER_LOTE;

            //Filler - Posição: 9 a 17 - Picture: X(9)
            $linha .= str_repeat(' ', 9);

            //Quantidade de Registro do lote - Posição: 18 a 23 - Picture: 9(6)
            $linha .= str_pad($this->qtdRegiLote, 6, '0', STR_PAD_LEFT);

            //Somatorio dos valores a serem lançados - Posição: 24 a 41 - Picture: 9(16)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($this->vrPagtLote,2)
            ),18,'0',STR_PAD_LEFT);

            //Somatório da quantidade de moeda - Posição: 42 a 59 - Picture: 9(18)
            $linha .= str_repeat('0', 18);

            //Número Aviso de Débito - Posição: 60 a 65 - Picture: 9(6)
            $linha .= str_repeat('0', 6);

            //Filler - Posição: 66 a 230 - Picture: X(165)
            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha .= str_repeat(' ', 175);

            //Zera o contador de registros do lote
            $this->qtdRegiLote = 0;

            //Zera o sequencial de registros do lote
            $this->seqRegLote = 0;

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }

    }

    /**
     * @return string
     * @throws \Exception
     */
    public function geraTrailerArquivo(){

        try { //Quantidade de registro do arquivo
            $this->qtdRegiArq++;

            //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
            $linha = self::CODIGO_COMPENSACAO_SANTANDER;

            //Número de sequência do lote - Posição: 4 a 7 - Picture: 9(4)
            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha .= self::LOTE_SERVICO_TRAILER_ARQUIVO . self::ID_TP_ARQUIVO_TRAILER;

            //Filler - Posição: 9 a 17 - Picture: X(9)
            $linha .= str_repeat(' ', 9);

            //Quantidade de lotes do arquivo - Posição: 18 a 23 - Picture: 9(6)
            $linha .= str_pad($this->qtLote, 6, '0', STR_PAD_LEFT);

            //Qtde total de registros do arquivo - Posição: 24 a 29 - Picture: 9(6)
            $linha .= str_pad($this->qtdRegiArq, 6, '0', STR_PAD_LEFT);

            //Filler - Posição: 30 a 240 - Picture: X(211)
            $linha .= str_repeat(' ', 211);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws \Exception
     */
    public function geraSegmentoJ(Boletpagar $boleto){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SANTANDER;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMENTO_J;

            //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
            if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha .= '9';  // Exclusão
            }
            else{
                $linha.=  '0';  // Inclusão e Bloqueio
                //Somatório dos registros com tipo de movimento = 0 (inclusão)
                $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            }


            //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
            if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
                $linha .= '00';  //INCLUSAO
            }
            else{
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.=  '99';  // EXLUSAO
                }
                else{//BLoqueio
                    $linha.=  '09';
                }
            }


            //Código de Barras - Posição: 18 a 61 - Picture: X(44)
            if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                if($boleto->getCdbarrapag()){
                    $cdbanco = str_pad(substr($boleto->getCdbarrapag(),0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($boleto->getCdbarrapag(),3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($boleto->getCdbarrapag(),4,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($boleto->getCdbarrapag(),5,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($boleto->getCdbarrapag(),19,25),25,'0',STR_PAD_LEFT);
                } else {
                    $linhaBarra = '';
                    $dslinhbarpag = $boleto->getDslinhbarpag();
                    for($i = 0;$i<strlen($dslinhbarpag);$i++){
                        if(in_array($dslinhbarpag[$i],array('0','1','2','3','4','5','6','7','8','9'))){
                            $linhaBarra.= $dslinhbarpag[$i];
                        }
                    }
                    $cdbanco = str_pad(substr($linhaBarra,0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($linhaBarra,3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($linhaBarra,32,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($linhaBarra,33,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($linhaBarra,4,25),25,'0',STR_PAD_LEFT);
                }
            }
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha.= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha.= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha.= $cddigito;

            //Valor (Cd. Barra) - Posição: 23 a 36 - Picture: 9(14)
            $linha.= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha.= $cdcampo;

            //Nome do cedente - Posição: 62 a 91 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

            //Data do vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do Título - Posição: 100 a 114 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Valor do Desconto + Abatimento - Posição: 115 a 129 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Valor do Acrescimo - Posição: 130 a 144 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Data do pagamento - Posição: 145 a 152 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do pagto - Posição: 153 a 167 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Quantidade de moeda - Posição:  168 a 182 - Picture: 9(15)
            $linha.= str_repeat('0',15);

            //Cód. Refer. atribuído pelo sacado - Posição: 183 a 202 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Número do compr. atribuído pelo banco - Posição: 203 a 222 - Picture: X(20)
            if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha.= str_pad(substr($boleto->getNrretcnabpag(),0,20),20,' ',STR_PAD_RIGHT);
            } else{
                $linha.= str_repeat(' ',20);
            }

            //Código da Moeda - Posição: 223 a 224 - Picture: X(02)
            $linha.= str_repeat('0',2);

            //Filler - Posição: 225 a 230 - Picture: X(06)
            $linha.= str_repeat(' ',6);

            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws \Exception
     */
    public function geraSegmentoJ52(Boletpagar $boleto){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SANTANDER;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad($this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMENTO_J;
            
            //Filler - Posição: 15 a 15 - Picture: X(01)
            $linha.= str_repeat(' ',1);

            //Código de Movimento Remessa - Posição: 16 a 17 - Picture: 9(02)
            $linha.= str_repeat('0',2);
            
            //Identificação Registro - Posição: 18 a 19 - Picture: 9(02)
            $linha.= self::SEGMENTO_J52;
            
            /*
             * #### PAGADOR ####
             */
            //Tipo de Inscrição Pagador - Posição: 20 a 20 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CPF/CNPJ do Pagador - Posição: 21 a 35 - Picture: 9(15)
            $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
            
            //Nome do Pagador - Posição: 36 a 75 - Picture: X(40)
            $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
            
            /*
             * #### Beneficiário ####
             */
            //Tipo de Inscrição Beneficiário - Posição: 76 a 76 - Picture: 9(01)
            if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
                $linha.= self::TIPO_INS_JUR_CPF;
            }else{
                $linha.= self::TIPO_INS_JUR_CNPJ;
            }
            
            //CPF/CNPJ do Beneficiário - Posição: 77 a 91 - Picture: 9(01)
            $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),15,'0', STR_PAD_LEFT);

            //Nome do Beneficiário - Posição: 92 a 131 - Picture: X(40)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,40)),40);
            
            /*
             * #### SACADOR ####
             */
            //Tipo de Inscrição Sacador - Posição: 132 a 132 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CPF/CNPJ do Sacador - Posição: 133 a 147 - Picture: 9(15)
            $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
            
            //Nome do Sacador - Posição: 148 a 187 - Picture: X(40)
            $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
            
            //Filler - Posição: 188 a 240 - Picture: X(53)
            $linha.= str_repeat(' ',53);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws \Exception
     */
    public function geraSegmentoO(Boletpagar $boleto){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SANTANDER;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMENTO_O;//COLOCAR

            //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
            if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha .= '9';  // Exclusão
            }
            else{
                $linha.=  '0';  // Inclusão e Bloqueio
                //Somatório dos registros com tipo de movimento = 0 (inclusão)
                $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            }

            //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
            if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
                $linha .= '00';  //INCLUSAO
            }
            else{
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.=  '99';  // EXLUSAO
                }
                else{//BLoqueio
                    $linha.=  '09';
                }
            }
            
            $tiposbaixapag = array(Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS, Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL, Tipobaixapag::TIPO_BAIXA_FGTS);

            //Código de Barras - Posição: 18 a 61 - Picture: X(44)
            if(in_array($boleto->getTpbaixapag()->getIdbaixautilp(), $tiposbaixapag)){
                if($boleto->getCdbarrapag()){
                    $cdbanco = str_pad(substr($boleto->getCdbarrapag(),0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($boleto->getCdbarrapag(),3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($boleto->getCdbarrapag(),4,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($boleto->getCdbarrapag(),5,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($boleto->getCdbarrapag(),19,25),25,'0',STR_PAD_LEFT);
                } else {
                    $linhaBarra = '';
                    $dslinhbarpag = $boleto->getDslinhbarpag();
                    for($i = 0;$i<strlen($dslinhbarpag);$i++){
                        if(in_array($dslinhbarpag[$i],array('0','1','2','3','4','5','6','7','8','9'))){
                            $linhaBarra.= $dslinhbarpag[$i];
                        }
                    }
                    $cdbanco = str_pad(substr($linhaBarra,0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($linhaBarra,3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($linhaBarra,32,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($linhaBarra,33,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($linhaBarra,4,25),25,'0',STR_PAD_LEFT);
                }
            }
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha.= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha.= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha.= $cddigito;

            //Valor (Cd. Barra) - Posição: 23 a 36 - Picture: 9(14)
            $linha.= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha.= $cdcampo;

            //Nome Concessionária - Posição: 62 a 91 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

            //Data de Vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Data do Pagamento - Posição: 100 a 107 - Picture: X(44)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do Pagamento - Posição: 108 a 122 - Picture: 9(15)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);

            //Número Documento do Cliente - Posição: 123 a 142 - Picture: 9(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Número Documento atribuido pelo Banco - Posição: 143 a 162 - Picture: X(20)
            $linha.= str_repeat(' ',20);

            //Posição: Filler 163 a 230 - Picture: X(68)
            $linha.= str_repeat(' ',68);

            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param Boletpagar $boleto
     * @throws Exception
     * @return string
     * @internal param $cdbanco
     */
    public function geraDetalheSegmentoI(Boletpagar $boleto){
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SANTANDER;

            //Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMETO_I;

            //Filler - Posição: 15 a 15 - Picture: 9(01)
            $linha.= str_repeat(' ',1);
            
            //Código de Movimento Remessa - Posição: 16 a 17 - Picture: 9(02)
            $linha.= '01';
            
            //Número do contrato  - Posição: 18 a 27 - Picture: 9(10)
            $linha.= str_repeat('0',10);
            
            //Número do documento - Nota Fiscal - Posição: 28 a 42 - Picture: X(15)
            $linha.= str_repeat(' ',15);
            
            //Data da Compra - Posição: 43 a 50 - Picture: 9(08)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
            $notafiscal = self::getNotaFiscal($contaPagar->getNrlanctonf(), $contaPagar->getCdfilial(), $contaPagar->getNrorg());
            $linha.= $notafiscal->getDtemissao()->format('dmY');
            
            //Regime de Encargos Financeiros - Posição: 51 a 51 - Picture: 9(01)
            $linha.= str_repeat('0',1);
            
            //Modalidade de Encargo - Posição: 52 a 53 - Picture: 9(02)
            $linha.= str_repeat('0',2);
            
            //Taxa de operação a Desconto - Posição: 54 a 63 - Picture: 9(03)v7
            $linha.= str_repeat('0',10); //verificar
            
            //Data do Primeiro vencimento - Posição: 64 a 71 - Picture: 9(08)
            $linha.= str_repeat('0',8);
            
            //Data do Vencimento Final - Posição: 72 a 79 - Picture: 9(08)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            //Tipo de Vencimento da Parcela - Posição: 80 a 80 - Picture: 9(01)
            $linha.= str_repeat('0',1);
            
            //Periodicidade Prazo de vencimento - Posição: 81 a 82 - Picture: 9(02)
            $linha.= str_repeat('0',2);
            
            //Quantidade de Parcelas - Posição: 83 a 84 - Picture: 9(02)
            $linha.= $boleto->getNrparcbolpag();
            
            //Nosso Número - Posição: 85 a 104 - Picture: X(20)
            $linha.= str_repeat(' ',20);
            
            //Forma de Pagamento - Posição: 105 a 105 - Picture: 9(01)
            $linha.= str_repeat('0',1);
            
            //Valor de encargos da operação - Posição: 106 a 120 - Picture: 9(13)v2
            $linha.= str_repeat('0',15);
            
            //Pagamento do IOF - Posição: 121 a 121 - Picture: 9(01)
            $linha.= str_repeat('0',1);
            
            //Valor do IOF - Posição: 122 a 136 - Picture: 9(15)v2
            $linha.= str_repeat('0',15);
            
            //Valor da Operação - Posição: 137 a 151 - Picture: 9(13)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
            
            //Juros de Mora - Posição: 152 a 166 - Picture: 9(13)v2
            $linha.= str_repeat('0',15);
            
            //Valor IOF sobre atraso - Posição: 167 a 181 - Picture: 9(13)v2
            $linha.= str_repeat(' ',15);
            
            //Multa - Posição: 182 a 196 - Picture: 9(13)v2
            $linha.= str_repeat(' ',15);
            
            //Filler - Posição: 197 a 240 - Picture: X(44)
            $linha.= str_repeat(' ',44);
            

            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }

    }

}